package org.zheng.demo;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.File;
import java.io.IOException;

import org.junit.jupiter.api.Test;
import org.zheng.demo.data.Customer;
import org.zheng.demo.data.Order;
import org.zheng.demo.data.Person;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.exc.MismatchedInputException;

class TestCustomer {

	private ObjectMapper ob = new ObjectMapper();

	@Test
	void test_customer_full_happyPath() {
		File file = new File("src/test/resources/customer_full.json");

		try {
			Customer readCust = ob.readValue(file, Customer.class);
			assertEquals("Mary", readCust.getPerson().getName());
			assertEquals(50, readCust.getPerson().getAge());
			assertEquals("test@test.com", readCust.getEmail());
			assertEquals("major", readCust.getDataType());
			assertEquals(2, readCust.getId());
			assertEquals(2, readCust.getOrders().size());
		} catch (JsonProcessingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Test
	void test_customer_no_email_throw_JsonMappingException() {
		File file = new File("src/test/resources/customer_no_email.json");

		JsonMappingException expectedException = assertThrows(JsonMappingException.class, () -> {
			ob.readValue(file, Customer.class);
		});

		assertTrue(expectedException.getMessage().contains("email is marked non-null but is null"));

	}

	@Test
	void test_customer_no_person_is_ok() {
		File file = new File("src/test/resources/customer_no_person.json");

		try {
			Customer cust = ob.readValue(file, Customer.class);
			assertNull(cust.getPerson());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Test
	void test_customer_null_email_throw_JsonMappingException() {
		File file = new File("src/test/resources/customer_null_email.json");

		JsonMappingException expectedException = assertThrows(JsonMappingException.class, () -> {
			ob.readValue(file, Customer.class);
		});

		assertTrue(expectedException.getMessage().contains("email is marked non-null but is null"));

	}

	@Test
	void test_customer_person_no_name_throw_MismatchedInputException() {
		File file = new File("src/test/resources/customer_person_no_name.json");
		MismatchedInputException expectedException = assertThrows(MismatchedInputException.class,
				() -> ob.readValue(file, Customer.class));
		assertTrue(expectedException.getMessage().contains("Missing required creator property 'person_name' "));

	}

	@Test
	void test_customer_wo_email_personName_throws_MismatchedInputException() {
		File file = new File("src/test/resources/customer_no_email_no_personName.json");

		MismatchedInputException expectedException = assertThrows(MismatchedInputException.class,
				() -> ob.readValue(file, Customer.class));

		assertTrue(expectedException.getMessage().contains("Missing required creator property 'person_name'"));

	}

	@Test
	void test_JsonProperty_field() {
		Customer customer = new Customer("test@test.com");
		customer.setDataType("major");
		customer.setId(2);
		
		Person pers = new Person();
		pers.setName("Zheng");
		pers.setAge(10);
		customer.setPerson(pers);
		Order order = new Order("test", 10);
		customer.addOrder(order);
		Order order2 = new Order("PS", 20);
		customer.addOrder(order2);

		try {
			String jsonStr = ob.writerWithDefaultPrettyPrinter().writeValueAsString(customer);
			System.out.println(jsonStr);

			Customer readCust = ob.readValue(jsonStr, Customer.class);

			assertTrue(customer.equals(readCust));
			assertEquals("major", readCust.getDataType());
			assertEquals("test@test.com", readCust.getEmail());
			assertEquals(2, readCust.getId());
			assertEquals(2, readCust.getOrders().size());
			assertEquals("test", readCust.getOrders().get(0).getProductName());
			assertEquals(10, readCust.getOrders().get(0).getQuantity());
		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}
	}

}
