package org.zheng.demo;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Test;
import org.zheng.demo.data.Person;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.exc.MismatchedInputException;

class TestPerson {

	private ObjectMapper ob = new ObjectMapper();

	@Test
	void test_happy_path() {
		// name and age have value
		String json = "{\"person_name\":\"John Doe\",\"age\":30}";
		try {
			Person pers = ob.readValue(json, Person.class);
			assertEquals("John Doe", pers.getName());
			assertEquals(30, pers.getAge());
		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}

	}

	@Test
	void test_required_throw_exception_if_not_there() {
		// name is not there,
		String jsonWOName = "{\"name1\":\"John Doe\",\"age\":30}";
		MismatchedInputException expectedException = assertThrows(MismatchedInputException.class, () -> ob.readValue(jsonWOName, Person.class));
		assertTrue(expectedException.getMessage().contains("Missing required creator property 'person_name'") );

	}

	@Test
	void test_required_with_null_is_ok() {
		// has name but the value is null
		String jsonNullName = "{\"person_name\":null,\"age\":30}";
		try {
			Person pers2 = ob.readValue(jsonNullName, Person.class);
			assertNull(pers2.getName());
			assertEquals(30, pers2.getAge());
			
			System.out.println(ob.writeValueAsString(pers2));
		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}
	}

}
